import { AlgorithmFactory } from './algorithm-factory.abstract';
import { ClassicWaypointsCompleter } from '../processor/completer/classic-waypoint.completer.processor';
import { RouteCompleter } from '../processor/completer/route.completer.processor';
import { ClassicGeoFilter } from '../processor/filter/geofilter/classic.filter.processor';
import { JourneyCompleter } from '../processor/completer/journey.completer.processor';
import { ClassicTimeFilter } from '../processor/filter/timefilter/classic.filter.processor';
import { Processor } from '../processor/processor.abstract';
import { Selector } from '../selector/selector.abstract';
import { ClassicSelector } from '../selector/classic.selector';

export class ClassicAlgorithmFactory extends AlgorithmFactory {
  createSelector = (): Selector => new ClassicSelector(this.matchQuery);
  createProcessors = (): Processor[] => [
    new ClassicWaypointsCompleter(this.matchQuery),
    new RouteCompleter(this.matchQuery, true, true, true),
    new ClassicGeoFilter(this.matchQuery),
    new RouteCompleter(this.matchQuery),
    new JourneyCompleter(this.matchQuery),
    new ClassicTimeFilter(this.matchQuery),
  ];
}
