import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { CqrsModule } from '@nestjs/cqrs';
import { DatabaseModule } from '../database/database.module';
import { MatcherController } from './adapters/primaries/matcher.controller';
import { MatchProfile } from './mappers/match.profile';
import { MatchUseCase } from './domain/usecases/match.usecase';
import { CacheModule } from '@nestjs/cache-manager';
import { RedisClientOptions } from '@liaoliaots/nestjs-redis';
import { redisStore } from 'cache-manager-ioredis-yet';
import { DefaultParamsProvider } from './adapters/secondaries/default-params.provider';
import { GeorouterCreator } from './adapters/secondaries/georouter-creator';
import { HttpModule } from '@nestjs/axios';
import { MatcherGeodesic } from './adapters/secondaries/geodesic';
import { Matcher } from './domain/entities/engine/matcher';
import { AlgorithmFactoryCreator } from './domain/entities/engine/factory/algorithm-factory-creator';
import { TimezoneFinder } from './adapters/secondaries/timezone-finder';
import { GeoTimezoneFinder } from '../geography/adapters/secondaries/geo-timezone-finder';
import { GeographyModule } from '../geography/geography.module';
import { TimeConverter } from './adapters/secondaries/time-converter';
import { MESSAGE_BROKER_PUBLISHER, MESSAGE_PUBLISHER } from 'src/app.constants';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { MessagePublisher } from './adapters/secondaries/message-publisher';

@Module({
  imports: [
    GeographyModule,
    DatabaseModule,
    CqrsModule,
    HttpModule,
    CacheModule.registerAsync<RedisClientOptions>({
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        store: await redisStore({
          host: configService.get<string>('REDIS_HOST'),
          port: configService.get<number>('REDIS_PORT'),
          password: configService.get<string>('REDIS_PASSWORD'),
          ttl: configService.get<number>('CACHE_TTL'),
        }),
      }),
      inject: [ConfigService],
    }),
  ],
  controllers: [MatcherController],
  providers: [
    MatchProfile,
    DefaultParamsProvider,
    MatchUseCase,
    GeorouterCreator,
    MatcherGeodesic,
    TimezoneFinder,
    TimeConverter,
    Matcher,
    AlgorithmFactoryCreator,
    GeoTimezoneFinder,
    {
      provide: MESSAGE_BROKER_PUBLISHER,
      useClass: MessageBrokerPublisher,
    },
    {
      provide: MESSAGE_PUBLISHER,
      useClass: MessagePublisher,
    },
  ],
  exports: [],
})
export class MatcherModule {}
