import { MatchRequest } from '../../../../domain/dtos/match.request';
import { AlgorithmType } from '../../../../domain/types/algorithm.enum';
import { IDefaultParams } from '../../../../domain/types/default-params.type';
import { MatchQuery } from '../../../../queries/match.query';
import { ClassicWaypointsCompleter } from '../../../../domain/entities/engine/processor/completer/classic-waypoint.completer.processor';
import { Candidate } from '../../../../domain/entities/engine/candidate';

const mockGeorouterCreator = {
  create: jest.fn().mockImplementation(),
};

const mockTimezoneFinder = {
  timezones: jest.fn().mockImplementation(() => ['Europe/Paris']),
};

const mockTimeConverter = {
  toUtcDate: jest.fn(),
};

const defaultParams: IDefaultParams = {
  DEFAULT_UUID: '00000000-0000-0000-0000-000000000000',
  MARGIN_DURATION: 900,
  VALIDITY_DURATION: 365,
  DEFAULT_TIMEZONE: 'Europe/Paris',
  DEFAULT_SEATS: 3,
  DEFAULT_ALGORITHM_SETTINGS: {
    ALGORITHM: AlgorithmType.CLASSIC,
    STRICT: false,
    REMOTENESS: 15000,
    USE_PROPORTION: true,
    PROPORTION: 0.3,
    USE_AZIMUTH: true,
    AZIMUTH_MARGIN: 10,
    MAX_DETOUR_DISTANCE_RATIO: 0.3,
    MAX_DETOUR_DURATION_RATIO: 0.3,
    GEOROUTER_TYPE: 'graphhopper',
    GEOROUTER_URL: 'http://localhost',
  },
};

const matchRequest: MatchRequest = new MatchRequest();
matchRequest.departure = '2023-04-01 12:00';
matchRequest.waypoints = [
  {
    lat: 49.440041,
    lon: 1.093912,
  },
  {
    lat: 50.630992,
    lon: 3.045432,
  },
];
const matchQuery: MatchQuery = new MatchQuery(
  matchRequest,
  defaultParams,
  mockGeorouterCreator,
  mockTimezoneFinder,
  mockTimeConverter,
);

describe('ClassicWaypointCompleter', () => {
  it('should be defined', () => {
    expect(new ClassicWaypointsCompleter(matchQuery)).toBeDefined();
  });

  it('should complete candidates', () => {
    const candidates = [new Candidate(), new Candidate()];
    const classicWaypointCompleter: ClassicWaypointsCompleter =
      new ClassicWaypointsCompleter(matchQuery);
    expect(classicWaypointCompleter.complete(candidates).length).toBe(2);
  });
});
