import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { CreateAdCommand } from './create-ad.command';
import { Inject } from '@nestjs/common';
import { AD_REPOSITORY } from '@modules/ad/ad.di-tokens';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { AggregateID, ConflictException } from '@mobicoop/ddd-library';
import { AdAlreadyExistsException } from '@modules/ad/core/domain/ad.errors';

@CommandHandler(CreateAdCommand)
export class CreateAdService implements ICommandHandler {
  constructor(
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
  ) {}

  async execute(command: CreateAdCommand): Promise<AggregateID> {
    const ad = AdEntity.create({
      id: command.id,
      driver: command.driver,
      passenger: command.passenger,
      frequency: command.frequency,
      fromDate: command.fromDate,
      toDate: command.toDate,
      schedule: command.schedule,
      seatsProposed: command.seatsProposed,
      seatsRequested: command.seatsRequested,
      strict: command.strict,
      waypoints: command.waypoints,
    });

    try {
      await this.repository.insert(ad);
      return ad.id;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new AdAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
