import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { CreateAdProps, Frequency } from '@modules/ad/core/domain/ad.types';
import { WaypointProps } from '@modules/ad/core/domain/value-objects/waypoint.value-object';

const originWaypointProps: WaypointProps = {
  position: 0,
  lon: 48.689445,
  lat: 6.17651,
};
const destinationWaypointProps: WaypointProps = {
  position: 1,
  lon: 48.8566,
  lat: 2.3522,
};

const createAdProps: CreateAdProps = {
  id: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
  driver: true,
  passenger: true,
  fromDate: '2023-06-21',
  toDate: '2023-06-21',
  schedule: [
    {
      day: 3,
      time: '08:30',
      margin: 900,
    },
  ],
  frequency: Frequency.PUNCTUAL,
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
  waypoints: [originWaypointProps, destinationWaypointProps],
};

describe('Ad entity create', () => {
  it('should create a new entity', async () => {
    const ad: AdEntity = AdEntity.create(createAdProps);
    expect(ad.id.length).toBe(36);
    expect(ad.getProps().schedule.length).toBe(1);
    expect(ad.getProps().schedule[0].day).toBe(3);
    expect(ad.getProps().schedule[0].time).toBe('08:30');
    expect(ad.getProps().driver).toBeTruthy();
    expect(ad.getProps().passenger).toBeTruthy();
    expect(ad.getProps().driverDistance).toBeUndefined();
  });
});
