import {
  AD_DIRECTION_ENCODER,
  AD_ROUTE_PROVIDER,
} from '@modules/ad/ad.di-tokens';
import { AdMapper } from '@modules/ad/ad.mapper';
import { RouteProviderPort } from '@modules/ad/core/application/ports/route-provider.port';
import { AdRepository } from '@modules/ad/infrastructure/ad.repository';
import { PrismaService } from '@modules/ad/infrastructure/prisma.service';
import { DirectionEncoderPort } from '@modules/geography/core/application/ports/direction-encoder.port';
import { EventEmitter2, EventEmitterModule } from '@nestjs/event-emitter';
import { Test, TestingModule } from '@nestjs/testing';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

const mockDirectionEncoder: DirectionEncoderPort = {
  encode: jest.fn(),
  decode: jest.fn(),
};

const mockRouteProvider: RouteProviderPort = {
  get: jest.fn(),
};

describe('Ad repository', () => {
  let prismaService: PrismaService;
  let adMapper: AdMapper;
  let eventEmitter: EventEmitter2;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [EventEmitterModule.forRoot()],
      providers: [
        PrismaService,
        AdMapper,
        {
          provide: AD_DIRECTION_ENCODER,
          useValue: mockDirectionEncoder,
        },
        {
          provide: AD_ROUTE_PROVIDER,
          useValue: mockRouteProvider,
        },
      ],
    }).compile();

    prismaService = module.get<PrismaService>(PrismaService);
    adMapper = module.get<AdMapper>(AdMapper);
    eventEmitter = module.get<EventEmitter2>(EventEmitter2);
  });
  it('should be defined', () => {
    expect(
      new AdRepository(
        prismaService,
        adMapper,
        eventEmitter,
        mockMessagePublisher,
      ),
    ).toBeDefined();
  });
});
