import { GeorouterPort } from '../application/ports/georouter.port';
import { GeorouterSettings } from '../application/types/georouter-settings.type';
import { SpacetimePointProps } from './value-objects/spacetime-point.value-object';
import { WaypointProps } from './value-objects/waypoint.value-object';

// All properties that a Route has
export interface RouteProps {
  driverDistance?: number;
  driverDuration?: number;
  passengerDistance?: number;
  passengerDuration?: number;
  fwdAzimuth: number;
  backAzimuth: number;
  distanceAzimuth: number;
  waypoints: WaypointProps[];
  spacetimePoints: SpacetimePointProps[];
}

// Properties that are needed for a Route creation
export interface CreateRouteProps {
  roles: Role[];
  waypoints: WaypointProps[];
  georouter: GeorouterPort;
  georouterSettings: GeorouterSettings;
}

export type Direction = {
  type: PathType;
  distance: number;
  duration: number;
  fwdAzimuth: number;
  backAzimuth: number;
  distanceAzimuth: number;
  points: Point[];
  spacetimePoints: SpacetimePoint[];
};

export type Path = {
  type: PathType;
  points: Point[];
};

export type Point = {
  lon: number;
  lat: number;
};

export type SpacetimePoint = Point & {
  duration: number;
  distance: number;
};

export enum Role {
  DRIVER = 'DRIVER',
  PASSENGER = 'PASSENGER',
}

export enum PathType {
  GENERIC = 'generic',
  DRIVER = 'driver',
  PASSENGER = 'passenger',
}
