import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import { DIRECTION_ENCODER, PARAMS_PROVIDER } from './geography.di-tokens';
import { DefaultParamsProvider } from './infrastructure/default-params-provider';
import { PostgresDirectionEncoder } from './infrastructure/postgres-direction-encoder';
import { GetRouteController } from './interface/controllers/get-route.controller';

const adapters: Provider[] = [
  {
    provide: PARAMS_PROVIDER,
    useClass: DefaultParamsProvider,
  },
  {
    provide: DIRECTION_ENCODER,
    useClass: PostgresDirectionEncoder,
  },
  GetRouteController,
];

@Module({
  imports: [CqrsModule],
  providers: [...adapters],
  exports: [DIRECTION_ENCODER, GetRouteController],
})
export class GeographyModule {}
