import { QueryBus } from '@nestjs/cqrs';
import { RouteResponseDto } from '../dtos/route.response.dto';
import { GetRouteRequestDto } from './dtos/get-route.request.dto';
import { RouteEntity } from '@modules/geography/core/domain/route.entity';
import { GetRouteQuery } from '@modules/geography/core/application/queries/get-route/get-route.query';
import { RouteMapper } from '@modules/geography/geography.mapper';
import { Injectable } from '@nestjs/common';

@Injectable()
export class GetRouteController {
  constructor(
    private readonly queryBus: QueryBus,
    private readonly mapper: RouteMapper,
  ) {}

  async get(data: GetRouteRequestDto): Promise<RouteResponseDto> {
    try {
      const route: RouteEntity = await this.queryBus.execute(
        new GetRouteQuery(data.roles, data.waypoints, data.georouterSettings),
      );
      return this.mapper.toResponse(route);
    } catch (e) {
      throw e;
    }
  }
}
